# Debugging expander profile for AKS clusters

Here's some resources that are helpful for debugging purposes.
This guide assumes that an AKS cluster has already been created and has two node pools configured with autoscaling enabled.

## Initial setup

Make sure the resource group and cluster name are set correctly!

```console
export RESOURCE_GROUP="andreas-test"
export CLUSTER_NAME="sum-aks-test"

az aks get-credentials -g ${RESOURCE_GROUP:?} -n ${CLUSTER_NAME:?} --admin
```

### Scale-down timeout

Optionally, set shorter timeout for cluster scale-down like so:

```console
az aks update -g ${RESOURCE_GROUP:?} -n ${CLUSTER_NAME:?} --cluster-autoscaler-profile scale-down-delay-after-add=1m scale-down-unready-time=1m scale-down-unneeded-time=1m
```

### Configuration

The cluster autoscaler priority expander is configured by deploying a `ConfigMap` in the `kube-system` namespace.
Ensure that the regex matches the vmss for each respective node pool.

Highest value wins, according to [the documentation](https://github.com/kubernetes/autoscaler/blob/master/cluster-autoscaler/expander/priority/readme.md#configuration).

```console
kubectl apply -f ./cluster-autoscaler-priority-expander.yaml
az aks update -g ${RESOURCE_GROUP:?} -n ${CLUSTER_NAME:?} --cluster-autoscaler-profile expander=priority
```

Now, run the following command and verify that the field `expander` is set to `priority`:

```console
$ az resource show --ids /subscriptions/a9843866-e1ba-46e1-8d6d-819bf60e48ae/resourcegroups/${RESOURCE_GROUP:?}/providers/Microsoft.ContainerService/managedClusters/${CLUSTER_NAME:?} --query properties.autoScalerProfile
{
  "balance-similar-node-groups": "false",
  "expander": "priority",
  "max-empty-bulk-delete": "10",
  "max-graceful-termination-sec": "600",
  "max-node-provision-time": "15m",
  "max-total-unready-percentage": "45",
  "new-pod-scale-up-delay": "0s",
  "ok-total-unready-count": "3",
  "scale-down-delay-after-add": "1m",
  "scale-down-delay-after-delete": "10s",
  "scale-down-delay-after-failure": "3m",
  "scale-down-unneeded-time": "1m",
  "scale-down-unready-time": "1m",
  "scale-down-utilization-threshold": "0.5",
  "scan-interval": "10s",
  "skip-nodes-with-local-storage": "false",
  "skip-nodes-with-system-pods": "true"
}
```

## Testing

The resource requests for the pods in the sleeper deployment is chosen so that there will not fit more than one pod per node.
This makes it very easy to scale up and down the number of nodes simply by installing the deployment and changing the number of replicas.

1. `kubectl apply -f ./sleeper-deployment.yaml`
1. `kubectl scale deployment sleeper --replicas 2` (Increase the number of replicas as desired)

You probably want to monitor what is happening in separate terminals. Some useful commands to keep an eye on are:

* `watch kubectl get nodes,pods -o wide`
* `kubectl get events -w | grep -i scale`

## Other notes

In some cases, notably when there has recently been a patch applied to AKS that changes cluster autoscaler behaviour, you may have to run the following commands to ensure that the patch is applied:

```console
$ K8S_VERSION=$(az aks show -g ${RESOURCE_GROUP:?} -n ${CLUSTER_NAME:?} | jq -r '.kubernetesVersion')
$ az aks upgrade -g ${RESOURCE_GROUP:?} -n ${CLUSTER_NAME:?} -k "${K8S_VERSION:?}" -y --no-wait
The cluster is already on version 1.19.11 and is not in a failed state. No operations will occur when upgrading to the same version if the cluster is not in a failed state.
```
